@@ -19,12 +19,16 @@
 
 import socket
 
-from . import nmclient
 from libnmstate.nm import dns as nm_dns
 from libnmstate.nm import route as nm_route
 from libnmstate.schema import InterfaceIPv4
 from libnmstate.schema import Route
 
+from ..ifaces import BaseIface
+from .common import NM
+
+INT32_MAX = 2 ** 31 - 1
+
 
 def create_setting(config, base_con_profile):
     setting_ipv4 = None
@@ -46,15 +50,13 @@ def create_setting(config, base_con_profile):
             setting_ipv4.props.dns_priority = nm_dns.DEFAULT_DNS_PRIORITY
 
     if not setting_ipv4:
-        setting_ipv4 = nmclient.NM.SettingIP4Config.new()
+        setting_ipv4 = NM.SettingIP4Config.new()
 
     setting_ipv4.props.dhcp_client_id = "mac"
-    setting_ipv4.props.method = nmclient.NM.SETTING_IP4_CONFIG_METHOD_DISABLED
+    setting_ipv4.props.method = NM.SETTING_IP4_CONFIG_METHOD_DISABLED
     if config and config.get(InterfaceIPv4.ENABLED):
         if config.get(InterfaceIPv4.DHCP):
-            setting_ipv4.props.method = (
-                nmclient.NM.SETTING_IP4_CONFIG_METHOD_AUTO
-            )
+            setting_ipv4.props.method = NM.SETTING_IP4_CONFIG_METHOD_AUTO
             setting_ipv4.props.ignore_auto_routes = not config.get(
                 InterfaceIPv4.AUTO_ROUTES, True
             )
@@ -64,26 +66,28 @@ def create_setting(config, base_con_profile):
             setting_ipv4.props.ignore_auto_dns = not config.get(
                 InterfaceIPv4.AUTO_DNS, True
             )
+            # NetworkManager will remove the virtual interfaces like bridges
+            # when the DHCP timeout expired, set it to the maximum value to
+            # make this unlikely.
+            setting_ipv4.props.dhcp_timeout = INT32_MAX
         elif config.get(InterfaceIPv4.ADDRESS):
-            setting_ipv4.props.method = (
-                nmclient.NM.SETTING_IP4_CONFIG_METHOD_MANUAL
-            )
+            setting_ipv4.props.method = NM.SETTING_IP4_CONFIG_METHOD_MANUAL
             _add_addresses(setting_ipv4, config[InterfaceIPv4.ADDRESS])
         nm_route.add_routes(
-            setting_ipv4, config.get(nm_route.ROUTE_METADATA, [])
+            setting_ipv4, config.get(BaseIface.ROUTES_METADATA, [])
         )
-        nm_dns.add_dns(setting_ipv4, config.get(nm_dns.DNS_METADATA, {}))
+        nm_dns.add_dns(setting_ipv4, config.get(BaseIface.DNS_METADATA, {}))
         nm_route.add_route_rules(
             setting_ipv4,
             socket.AF_INET,
-            config.get(nm_route.ROUTE_RULES_METADATA, []),
+            config.get(BaseIface.ROUTE_RULES_METADATA, []),
         )
     return setting_ipv4
 
 
 def _add_addresses(setting_ipv4, addresses):
     for address in addresses:
-        naddr = nmclient.NM.IPAddress.new(
+        naddr = NM.IPAddress.new(
             socket.AF_INET,
             address[InterfaceIPv4.ADDRESS_IP],
             address[InterfaceIPv4.ADDRESS_PREFIX_LENGTH],
@@ -105,7 +109,7 @@ def get_info(active_connection):
     ip_profile = get_ip_profile(active_connection)
     if ip_profile:
         info[InterfaceIPv4.DHCP] = ip_profile.get_method() == (
-            nmclient.NM.SETTING_IP4_CONFIG_METHOD_AUTO
+            NM.SETTING_IP4_CONFIG_METHOD_AUTO
         )
         props = ip_profile.props
         if info["dhcp"]:
@@ -149,24 +153,24 @@ def get_ip_profile(active_connection):
     return None
 
 
-def get_route_running():
-    return nm_route.get_running(_acs_and_ip_cfgs(nmclient.client()))
+def get_route_running(context):
+    return nm_route.get_running(_acs_and_ip_cfgs(context))
 
 
-def get_route_config():
-    return nm_route.get_config(acs_and_ip_profiles(nmclient.client()))
+def get_route_config(context):
+    return nm_route.get_config(acs_and_ip_profiles(context))
 
 
-def _acs_and_ip_cfgs(client):
-    for ac in client.get_active_connections():
+def _acs_and_ip_cfgs(nm_client):
+    for ac in nm_client.get_active_connections():
         ip_cfg = ac.get_ip4_config()
         if not ip_cfg:
             continue
         yield ac, ip_cfg
 
 
-def acs_and_ip_profiles(client):
-    for ac in client.get_active_connections():
+def acs_and_ip_profiles(nm_client):
+    for ac in nm_client.get_active_connections():
         ip_profile = get_ip_profile(ac)
         if not ip_profile:
             continue
@@ -176,13 +180,9 @@ def acs_and_ip_profiles(client):
 def is_dynamic(active_connection):
     ip_profile = get_ip_profile(active_connection)
     if ip_profile:
-        return ip_profile.get_method() == (
-            nmclient.NM.SETTING_IP4_CONFIG_METHOD_AUTO
-        )
+        return ip_profile.get_method() == NM.SETTING_IP4_CONFIG_METHOD_AUTO
     return False
 
 
-def get_routing_rule_config():
-    return nm_route.get_routing_rule_config(
-        acs_and_ip_profiles(nmclient.client())
-    )
+def get_routing_rule_config(nm_client):
+    return nm_route.get_routing_rule_config(acs_and_ip_profiles(nm_client))
